/*
*
* Copyright 2012  Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
*/

#ifndef __EXTENDED_EDC_H__
#define __EXTENDED_EDC_H__

#define PPART PART:
#define PIMAGE IMAGE:

/***********************************************************************/
/* size difinition layer */
#define PART_BG(iName)\
part {\
	name: iName;\
	type: RECT;\
	mouse_events: 0;\
	scale: 1;\
	description {\
		state: "default" 0.0;\
		color: 0 0 0 0;\
	} \
} \

#define PART_BG_EX(iName, iMinx, iMiny)\
part {\
	name: iName;\
	type: RECT;\
	mouse_events: 0;\
	scale: 1;\
	description {\
		state: "default" 0.0;\
		min: iMinx iMiny;\
		color: 0 0 0 0;\
	} \
} \

#define PART_RECT(iName, iR1x, iR1y, iR1t, iR2x, iR2y, iR2t)\
part {\
	name: iName;\
	type: RECT;\
	mouse_events: 0;\
	scale: 1;\
	description {\
		state: "default" 0.0;\
		rel1 { relative: iR1x iR1y; to: iR1t; } \
		rel2 { relative: iR2x iR2y; to: iR2t; } \
		color: 0 0 0 0;\
	} \
} \

#define PART_V_PIXEL_RECT(iName, size, iTo, iRy, iAy)\
part {\
	name: iName;\
	type: RECT;\
	mouse_events: 0;\
	scale: 1;\
	description {\
		state: "default" 0.0;\
		min: 0 size;\
		align: 0.5 iAy;\
		fixed: 0 1;\
		rel1 { relative: 0.0 iRy; to: iTo; } \
		rel2 { relative: 1.0 iRy; to: iTo; } \
		color: 0 0 0 0;\
	} \
} \

#define PART_H_PIXEL_RECT(iName, size, iTo, iRx, iAx)\
part {\
	name: iName;\
	type: RECT;\
	mouse_events: 0;\
	scale: 1;\
	description {\
		state: "default" 0.0;\
		min: size 0;\
		align: iAx 0.5;\
		fixed: 1 0;\
		rel1 { relative: iRx 0.0; to: iTo; } \
		rel2 { relative: iRx 1.0; to: iTo; } \
		color: 0 0 0 0;\
	} \
} \

#define PART_PIXEL_RECT(iName, iX, iY, iRx, iRy, iTo, iAx, iAy)\
part {\
	name: iName;\
	type: RECT;\
	mouse_events: 0;\
	scale: 1;\
	description {\
		state: "default" 0.0;\
		min: iX iY;\
		align: iAx iAy;\
		fixed: 1 1;\
		rel1 { relative: iRx iRy; to: iTo; } \
		rel2 { relative: iRx iRy; to: iTo; } \
		color: 0 0 0 0;\
	} \
} \


/* vertical divided rect, index range from 0~N-1
 * iIndex must be immediate integer and expression like (1+2) is not allowed
 */
#define PART_V_DIVID_RECT(iName, iTo, iIndex, iN)\
part {\
	name: iName;\
	type: RECT;\
	mouse_events: 0;\
	scale: 1;\
	description {\
		state: "default" 0.0;\
		rel1 { relative: 0.0 (iIndex%iN)/iN; to: iTo; } \
		rel2 { relative: 1.0 (iIndex%iN+1)/iN; to: iTo; } \
		color: 0 0 0 0;\
	} \
} \

/* horizontal divided rect, index range from 0~N-1
 * iIndex must be immediate integer and expression like (1+2) is not allowed
 */
#define PART_H_DIVID_RECT(iName, iTo, iIndex, iN)\
part {\
	name: iName;\
	type: RECT;\
	mouse_events: 0;\
	scale: 1;\
	description {\
		state: "default" 0.0;\
		rel1 { relative: (iIndex%iN)/iN 0.0; to: iTo; } \
		rel2 { relative: (iIndex%iN+1)/iN 1.0; to: iTo; } \
		color: 0 0 0 0;\
	} \
} \

#define PART_CLIPPER_RECT(iName, iR1x, iR1y, iR1t, iR2x, iR2y, iR2t)\
part {\
	name: iName;\
	type: RECT;\
	mouse_events: 0;\
	scale: 1;\
	description {\
		state: "default" 0.0;\
		rel1 { relative: iR1x iR1y; to: iR1t; } \
		rel2 { relative: iR2x iR2y; to: iR2t; } \
		color: 255 255 255 255;\
	} \
} \

/***********************************************************************/
/* mouse event filter layer */
#define PART_EVENT_RECT(iName, iTo)\
part {\
	name: iName;\
	type: RECT;\
	mouse_events: 1;\
	scale: 1;\
	description {\
		state: "default" 0.0;\
		rel1 { relative: 0.0 0.0; to: iTo; } \
		rel2 { relative: 1.0 1.0; to: iTo; } \
		color: 0 0 0 0;\
	} \
} \

/***********************************************************************/
/* content layer */
#define PART_COLOR_RECT(iName, iTo, iCr, iCg, iCb, iCa)\
part {\
	name: iName;\
	type: RECT;\
	mouse_events: 0;\
	scale: 1;\
	description {\
		state: "default" 0.0;\
		rel1 { relative: 0.0 0.0; to: iTo; } \
		rel2 { relative: 1.0 1.0; to: iTo; } \
		color: iCr iCg iCb iCa;\
	} \
} \

#define PART_COLOR_RECT_REL(iName, iR1x, iR1y, iR1t, iR2x, iR2y, iR2t, iCr, iCg, iCb, iCa)\
part {\
	name: iName;\
	type: RECT;\
	mouse_events: 0;\
	scale: 1;\
	description {\
		state: "default" 0.0;\
		rel1 { relative: iR1x iR1y; to: iR1t; } \
		rel2 { relative: iR2x iR2y; to: iR2t; } \
		color: iCr iCg iCb iCa;\
	} \
} \

#define PART_IMAGE(iName, iTo, iImage)\
part {\
	name: iName;\
	type: IMAGE;\
	mouse_events: 0;\
	scale: 1;\
	description {\
		state: "default" 0.0;\
		rel1 { relative: 0.0 0.0; to: iTo; } \
		rel2 { relative: 1.0 1.0; to: iTo; } \
		image.normal : iImage;\
	} \
} \

#define PART_IMAGE_REL(iName, iImage, iR1x, iR1y, iR1t, iR2x, iR2y, iR2t)\
PART_RECT("_"iName, iR1x, iR1y, iR1t, iR2x, iR2y, iR2t)\
PART_IMAGE(iName, "_"iName, iImage)\

#define PART_BORDER_IMAGE(iName, iTo, iImage, iBl, iBr, iBt, iBb)\
part {\
	name: iName;\
	type: IMAGE;\
	mouse_events: 0;\
	scale: 1;\
	description {\
		state: "default" 0.0;\
		rel1 { relative: 0.0 0.0; to: iTo; } \
		rel2 { relative: 1.0 1.0; to: iTo; } \
		image.normal : iImage;\
		image.border : iBl iBr iBt iBb;\
	} \
} \

#define PART_SWALLOW(iName, iTo)\
part {\
	name: iName;\
	type: SWALLOW;\
	mouse_events: 1;\
	scale: 1;\
	description {\
		state: "default" 0.0;\
		rel1 { relative: 0.0 0.0; to: iTo; } \
		rel2 { relative: 1.0 1.0; to: iTo; } \
	} \
} \

#define PART_SWALLOW_REL(iName, iR1x, iR1y, iR1t, iR2x, iR2y, iR2t)\
PART_RECT("_"iName, iR1x, iR1y, iR1t, iR2x, iR2y, iR2t)\
PART_SWALLOW(iName, "_"iName)\

#define PART_UNVISIBLE_SWALLOW(iName, iTo)\
part {\
	name: iName;\
	type: SWALLOW;\
	mouse_events: 1;\
	scale: 1;\
	description {\
		state: "default" 0.0;\
		visible: 0;\
		rel1 { relative: 0.0 0.0; to: iTo; } \
		rel2 { relative: 1.0 1.0; to: iTo; } \
	} \
} \

#define PART_UNVISIBLE_SWALLOW_REL(iName, iR1x, iR1y, iR1t, iR2x, iR2y, iR2t)\
PART_RECT("_"iName, iR1x, iR1y, iR1t, iR2x, iR2y, iR2t)\
PART_UNVISIBLE_SWALLOW(iName, "_"iName)\

#define PART_CLIPPED_SWALLOW(iName, iR1x, iR1y, iR1t, iR2x, iR2y, iR2t,\
		iAx, iAy, iCliper)\
part {\
	name: iName;\
	type: SWALLOW;\
	mouse_events: 0;\
	scale: 1;\
	clip_to: iCliper;\
	description {\
		state: "default" 0.0;\
		align: iAx iAy;\
		fixed: 1 1;\
		rel1 { relative: iR1x iR1y; to: iR1t; } \
		rel2 { relative: iR2x iR2y; to: iR2t; } \
	} \
} \

#define PART_TEXT(iName, iTo, iText, iFont, iClass, iSize,\
		iAx, iAy, iCr, iCg, iCb, iCa)\
part {\
	name: iName;\
	type: TEXT;\
	mouse_events: 0;\
	scale: 1;\
	description {\
		state: "default" 0.0;\
		rel1 { relative: 0.0 0.0; to: iTo; } \
		rel2 { relative: 1.0 1.0; to: iTo; } \
		color: iCr iCg iCb iCa;\
		text {\
			text: iText;\
			font: iFont;\
			text_class: iClass;\
			size: iSize;\
			align: iAx iAy;\
		} \
	} \
} \

#define PART_TEXT_REL(iName, iR1x, iR1y, iR1t, iR2x, iR2y, iR2t,\
		iText, iFont, iClass, iSize, iAx, iAy, iCr, iCg, iCb, iCa)\
PART_RECT("_"iName, iR1x, iR1y, iR1t, iR2x, iR2y, iR2t)\
PART_TEXT(iName, "_"iName, iText, iFont, iClass, iSize, iAx, iAy, iCr, iCg, iCb, iCa)\

/***********************************************************************/
/* script */
/* ugh!!!, iPart, iR1t, iR2t and iV of SCRIPT_SET_IMAGE should not be quoted with "" */
#define SCRIPT_CUSTOM_PART(iPart)\
custom_state(PPART#iPart, "default", 0.0);\
set_state(PPART#iPart, "custom", 0.0)

/* pay attention, rel paramater can only be float, script will not do evaluation of mathematics expression
	eg. 10/100 is illegal, 0.1 is correct */
#define SCRIPT_SET_REL(iPart, iR1x, iR1y, iR1t, iR2x, iR2y, iR2t)\
set_state_val(PPART#iPart, STATE_REL1, iR1x, iR1y);\
set_state_val(PPART#iPart, STATE_REL1_TO, PPART#iR1t, PPART#iR1t);\
set_state_val(PPART#iPart, STATE_REL2, iR2x, iR2y);\
set_state_val(PPART#iPart, STATE_REL2_TO, PPART#iR2t, PPART#iR2t)\

#define SCRIPT_SET_VISIBLE(iPart, iV)\
set_state_val(PPART#iPart, STATE_VISIBLE, iV)

#define SCRIPT_SET_IMAGE(iPart, iV)\
set_state_val(PPART#iPart, STATE_IMAGE, PIMAGE#iV)

#define SCRIPT_SET_COLOR(iPart, iCr, iCg, iCb, iCa)\
set_state_val(PPART#iPart, STATE_COLOR, iCr, iCg, iCb, iCa)


/***********************************************************************/
/* program */
/* ugh!!! isignal, isource, iPart should not quoted with ""  */
#define PROGRAM_SET_VISIBLE(isignal, isource, iPart, iV)\
program {\
	name: #isource#isignal;\
	source: #isource;\
	signal: #isignal;\
	script {\
		SCRIPT_SET_VISIBLE(iPart, iV);\
	} \
} \

#endif				/* __EXTENDED_EDC_H__ */
