/*
*
* Copyright 2012  Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
*/

#ifndef __GRAVEL_H__
#define __GRAVEL_H__

#include <dlog.h>

/* use prefix 'P' means print */
#define PTAG LOGD("[%s : %d]\n", __FILE__, __LINE__)
#define PLOG(fmt, arg...) LOGD(" ## "fmt, ##arg)
#define PFUNC_ENTER LOGD("ENTER : %s\n", __func__)
#define PFUNC_LEAVE LOGD("LEAVE : %s\n", __func__)

#define ARRAY_SIZE(array) \
	((int)(sizeof(array) / sizeof(array[0])))

#define STR_EQUAL(str1, str2) \
	(strcmp(str1, str2) == 0)

#define SSTRLEN(str)\
	(str == NULL ? 0 : strlen(str))\

#define sncat(to, size, from) \
	strncat(to, from, size-strlen(to)-1)

#define sncatprintf(s, size, fmt, args...) \
	snprintf(s+strlen(s), size-strlen(s), fmt, ##args)

#define SMALLOC(Type)\
	(Type *)calloc(1, sizeof(Type))\

#define SFREE(var)\
	if (var != NULL) {\
		free(var);\
		var = NULL;\
	} \

#define RETIF(condition)\
	if (condition) return

#define RETVIF(condition, val)\
	if (condition) return val

#endif				/* __GRAVEL_H__ */
