/*
*
* Copyright 2012  Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
*/

#include <dlog.h>
#include <gravel.h>
#include <supplement.h>

static bool _service_iterate_dump_cb(service_h service, const char *key, void *user_data)
{
    LOGD("%s -> %s\n", key, user_data);
    return 0;
}

void service_dump(service_h service)
{
    if (service != NULL) {
        service_foreach_extra_data(service, _service_iterate_dump_cb, NULL);
    }
}

Eina_Bool service_key_check(service_h service, const char *key, const char *val)
{
	char *str = NULL;
	service_get_extra_data(service, key, &str);
	if (str != NULL) {
		if (strcmp(str, val) == 0) {
			SFREE(str);
			return EINA_TRUE;
		}
		SFREE(str);
	}
	return EINA_FALSE;
}

void evas_object_geometry_dump(Evas_Object *eo)
{
    Evas_Coord x, y, w, h;
    evas_object_geometry_get(eo, &x, &y, &w, &h);
    LOGD("[evas_object_geometry_dump] geometry of %x : %d %d %d %d\n", eo, x, y, w, h);
}

void evas_object_event_hit(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
    LOGD("[%s] %x : %s\n", __func__, obj, (char *)data);
}

void evas_object_smart_event_hit(void *data, Evas_Object *obj, void *event_info)
{
    LOGD("[%s] %x : %s\n", __func__, obj, (char *)data);
}

typedef struct __rf_data_t {
    Evas_Smart_Cb cb;
    void *data;
}rf_data_t;

static void _render_flush_post_cb(void *data, Evas *e, void *event_info)
{
    rf_data_t *rf = (rf_data_t *)data;
    rf->cb(rf->data, NULL, NULL);
    evas_event_callback_del(e, EVAS_CALLBACK_RENDER_FLUSH_POST, _render_flush_post_cb);
    SFREE(rf);
}

void evas_object_render_flush_hook(Evas_Object *obj, Evas_Smart_Cb cb, void *data)
{
    RETIF(obj==NULL);
    Evas *e = evas_object_evas_get(obj);
    rf_data_t *rf = SMALLOC(rf_data_t);
    RETIF(rf==NULL);
    rf->cb = cb;
    rf->data = data;
    evas_event_callback_add(e, EVAS_CALLBACK_RENDER_FLUSH_POST, _render_flush_post_cb, rf);
}

