/*
*
* Copyright 2012  Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
*/

#ifndef __MEMO_ASSIST_H__
#define __MEMO_ASSIST_H__

#include <memo-db.h>
#include <bundle.h>
#include <memo_string.h>
#include <ui-gadget.h>


#define DOODLE_TITLE_PREFIX     "Draw_"

#define ARGB_JOIN(a, r, g, b) \
    (((a) << 24) + ((r) << 16) + ((g) << 8) + (b))

#define  IN_RECT_CHECK(x, y, rx, ry, rw, rh)\
    (((unsigned)((x) - (rx)) < (rw)) && ((unsigned)((y) - (ry)) < (rh)))

#define CLIP_RECT_TO_RECT(x, y, w, h, rx, ry, rw, rh)\
{\
    int   _t0, _t1;\
\
    _t0 = MAX(x, (rx));\
    _t1 = MIN(x + w, (rx) + (rw));\
    x = _t0;\
    w = _t1 - _t0;\
    _t0 = MAX(y, (ry));\
    _t1 = MIN(y + h, (ry) + (rh));\
    y = _t0;\
    h = _t1 - _t0;\
}

typedef void (*Memo_Component_Callback)(void *data, const char *msg, void *event);

typedef struct memo_data memo_data_t;
typedef struct memo_data_list memo_data_list_t;

typedef enum {
    MOUSE_EVENT_INVALID = 0,
    MOUSE_EVENT_CLICK,
    MOUSE_EVENT_DRAG_LEFT,
    MOUSE_EVENT_DRAG_RIGHT,
    MOUSE_EVENT_DRAG_UP,
    MOUSE_EVENT_DRAG_DOWN,
} mouse_behaviour_t;

#define MOUSE_POSITION_INVALID 9999

char * strcasestr (const char *haystack, const char *needle);
char * strptime (const char *s, const char *fmt, struct tm *tp);
void icu_init();
char *memo_get_doodle_title();
void memo_time_format(char *buf, int len, time_t time);
time_t memo_get_binary_release_date();
mouse_behaviour_t mouse_event_identify(int startx, int starty, int endx, int endy);
void evas_object_flick_callback_add(Evas_Object *obj, Evas_Smart_Cb on_flick, void *data);
void memo_com_dummy_cb(void *data, const char *msg, void *event);
void memo_create_yes_no_popup(Evas_Object *win, const char *msg, Evas_Smart_Cb on_yes, Evas_Smart_Cb on_no, void *data);
void memo_tool_btn_focus_set(Evas_Object *eo, Eina_Bool status);
void *memo_load_font_size_selector(Evas_Object *parent, service_h service, Memo_Component_Callback cb, void *data);
void memo_del_font_size_selector(void *h_fss);
void *memo_load_color_selector(Evas_Object *parent, service_h service, Memo_Component_Callback cb, void *data);
void memo_del_color_selector(void *h_cs);
void *memo_load_pencil_size_selector(Evas_Object *parent, service_h service, Memo_Component_Callback cb, void *data);
void memo_del_pencil_size_selector(void *h_pss);
void memo_navigator_pop(Evas_Object *navi);
Elm_Object_Item *memo_naviframe_item_push(Evas_Object *obj, const char *title_label,
    Evas_Object *prev_btn, Evas_Object *next_btn, Evas_Object *content, const char *item_style);
Evas_Object *memo_controlbar_add(Evas_Object *parent);
void memo_comment_recover(Evas_Object *entry, const char *orig_utf8);
#endif                /* __MEMO_ASSIST_H__ */
