/*
*
* Copyright 2012  Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
*/

#include <stdlib.h>
#include <stdio.h>
#include <Elementary.h>
#include <dlog.h>
#include <ui-gadget-module.h>
#include <supplement.h>
#include <memo_string.h>
#include <gravel.h>
#include <extended-elm.h>
#include <memo-assist.h>
#include <memo_text_editor.h>
#include <memo_doodle_editor.h>
#include <memo-efl.h>
#include <memo-genlist.h>

typedef struct __edit_view_t {
    ug_data_t *ugd;
    Evas_Object *navigator;
    Elm_Object_Item *navi_it;
    Evas_Object *ctrl_bar;
    Evas_Object *body_main;    /* content */
    Evas_Object *l_btn;
    Evas_Object *r_btn;
    Eina_Bool drawing;    /* drawing or text mode */

    /* add more variables here */
    void *h_ed;
} edit_view_t;

static void _text_editor_callback(void *data, const char *msg, void *event);
static void _doodle_editor_callback(void *data, const char *msg, void *event);
extern service_h text_service;

static void _on_left_btn_clicked(void *data, Evas_Object *obj, void *event_info)
{
    edit_view_t *ev = (edit_view_t *)data;
    if (ev->drawing) {
        memo_doodle_editor_save(ev->h_ed);
    } else {
        memo_text_editor_save(ev->h_ed);
    }
}

static void _on_right_btn_clicked(void *data, Evas_Object *obj, void *event_info)
{
    edit_view_t *ev = (edit_view_t *)data;
    if (ev->drawing) {
        memo_doodle_editor_cancel(ev->h_ed);
    } else {
        memo_text_editor_cancel(ev->h_ed);
    }
}
static void _text_editor_callback(void *data, const char *msg, void *event)
{
	edit_view_t *ev = (edit_view_t *)data;
	if (strcmp(msg, "layout") == 0) {
		elm_object_part_content_set(ev->body_main, "elm.swallow.content", (Evas_Object *)event);
	} else if (strcmp(msg, "title") == 0) {
		elm_object_item_text_set(ev->navi_it, (const char *)event);
	} else if (strcmp(msg, "savable") == 0) {
		elm_object_disabled_set(ev->r_btn, (int)event==0? EINA_TRUE:EINA_FALSE);
	} else if (strcmp(msg, "save") == 0) {
		ug_send_result(ev->ugd->ug,text_service);
		service_destroy(text_service);
		ug_destroy_me(ev->ugd->ug);
	} else if (strcmp(msg, "cancel") == 0) {
		ug_destroy_me(ev->ugd->ug);
	} else if (strcmp(msg, "drawing") == 0) {
		elm_object_part_content_unset(ev->body_main, "elm.swallow.content");
		memo_destroy_text_editor(ev->h_ed);
		ev->h_ed = memo_load_doodle_editor(ug_get_window(), ev->body_main, NULL, _doodle_editor_callback, ev);
		ev->drawing = EINA_TRUE;
	}
}

static void _doodle_editor_callback(void *data, const char *msg, void *event)
{
    edit_view_t *ev = (edit_view_t *)data;
    if (strcmp(msg, "layout") == 0) {
        elm_object_part_content_set(ev->body_main, "elm.swallow.content", (Evas_Object *)event);
    } else if (strcmp(msg, "title") == 0) {
        elm_object_item_text_set(ev->navi_it, (const char *)event);
    } else if (strcmp(msg, "savable") == 0) {
        elm_object_disabled_set(ev->r_btn, (int)event==0? EINA_TRUE:EINA_FALSE);
    } else if (strcmp(msg, "save") == 0) {
        ug_destroy_me(ev->ugd->ug);
    } else if (strcmp(msg, "cancel") == 0) {
        ug_destroy_me(ev->ugd->ug);
    } else if (strcmp(msg, "text") == 0) {
        elm_object_part_content_unset(ev->body_main, "elm.swallow.content");
        memo_destroy_doodle_editor(ev->h_ed);
        ev->h_ed = memo_load_text_editor(ug_get_window(), ev->body_main, NULL, _text_editor_callback, ev);
        ev->drawing = EINA_FALSE;
    }
}

static void _create_edit_layout(edit_view_t *ev, service_h data)
{
	ug_data_t *ugd = ev->ugd;

	/* add navigationbar */
	ev->navigator = elm_naviframe_add(ugd->base);
	elm_object_part_content_set(ugd->base, "elm.swallow.content", ev->navigator);

	/* custom body main here */
	ev->body_main = elm_layout_create(ev->navigator, EDJ_FILE, "empty_layout");
	/* compose */
	elm_naviframe_prev_btn_auto_pushed_set(ev->navigator, EINA_FALSE);
	Elm_Object_Item *navi_it = elm_naviframe_item_push(ev->navigator, MEMO_I18N_MEMO, NULL, NULL, ev->body_main, NULL);
	ev->navi_it = navi_it;
	/* buttons */
	ev->r_btn = elm_button_create(ev->navigator, MEMO_I18N_CANCEL,
									_on_right_btn_clicked, ev);
    elm_object_style_set(ev->r_btn, "naviframe/back_btn/default");
	elm_object_item_part_content_set(navi_it, "title_right_btn", ev->r_btn);
	ev->l_btn = elm_button_create(ev->navigator, MEMO_I18N_DONE,
										_on_left_btn_clicked, ev);
    elm_object_style_set(ev->l_btn, "naviframe/title/icon/edit");
	elm_object_item_part_content_set(navi_it, "title_left_btn", ev->l_btn);

	/* load editor */
	service_h service = NULL;
	service_create(&service);

	service_add_extra_data(service, "toggle", "disable");
	if (service_key_check(data, "type", "draw")) {
		ev->h_ed = memo_load_doodle_editor(ug_get_window(), ev->body_main, service, _doodle_editor_callback, ev);
		ev->drawing = EINA_TRUE;
	} else {
		char *str = NULL;
		service_get_extra_data(data, "text", &str);
		if (str != NULL) { /* set init string */
			service_add_extra_data(service, "init_str", str);
			SFREE(str);
		}
		ev->h_ed = memo_load_text_editor(ug_get_window(), ev->body_main, service, _text_editor_callback, ev);
		ev->drawing = EINA_FALSE;
	}
	service_destroy(service);
}

Eina_Bool edit_view_pre_condition_check(service_h service)
{
	char *str = NULL;
	service_get_extra_data(service, "type", &str);
	if (str != NULL) {
		if (!strcmp(str, "edit")) {
			SFREE(str);
			return EINA_TRUE;
		} else if (!strcmp(str, "draw")) {
			SFREE(str);
			return EINA_TRUE;
		}
		SFREE(str);
	}
	return EINA_FALSE;
}

void *memo_load_edit_view(ug_data_t *ugd, service_h service)
{
    edit_view_t *ev = SMALLOC(edit_view_t);
    RETVIF(ev == NULL, NULL);
    ev->ugd = ugd;

    _create_edit_layout(ev, service);
    return (void *)ev;
}

void memo_del_edit_view(void *h_ev)
{
    edit_view_t *ev = (edit_view_t *) h_ev;
    elm_object_part_content_unset(ev->body_main, "elm.swallow.content");
    if (ev->drawing) {
        memo_destroy_doodle_editor(ev->h_ed);
    } else {
        memo_destroy_text_editor(ev->h_ed);
    }
    evas_object_del(ev->navigator);
    SFREE(ev);
}

