/*
*
* Copyright 2012  Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
*/

#include <stdio.h>
#include <unicode/ustring.h>
#include <unicode/utypes.h>
#include <unicode/udat.h>
#include <unicode/udatpg.h>
#include <iniparser.h>
#include <dictionary.h>
#include <vconf-keys.h>
#include <vconf.h>
#include <gravel.h>
#include <extended-elm.h>
#include <supplement.h>
#include <memo-assist.h>
#include <appcore-common.h>

#define ICU_TIME_BUFFER 256
static UDateFormat* dfmt = NULL;

/**
 * memo_get_doodle_title
 *
 * @brief
 *
 * @return
 *
 * @exception    None
 *
 * @remark       None
 *
 * @see
 *
 */
char *memo_get_doodle_title()
{
    memo_data_list_t *l = NULL;
    memo_data_list_t *t = NULL;
    char buf[MEMO_BUFFER_SIZE];
    int i = 0;
    int d = 0;
    l = memo_get_all_data_list();
    for (t = l; t != NULL; t = t->next) {
        if (t->md.has_doodle) {
            d = atoi(t->md.content + strlen(DOODLE_TITLE_PREFIX));
            i = (d > i) ? d : i;
        }
    }
    i++;
    snprintf(buf, MEMO_BUFFER_SIZE, "%s%04d", DOODLE_TITLE_PREFIX, i);
    memo_free_data_list(l);
    return strdup(buf);
}

void icu_init()
{
    UErrorCode status = U_ZERO_ERROR;
    const char *locale;
    UChar ubuf[ICU_TIME_BUFFER];
    UChar bestPattern[ICU_TIME_BUFFER];
    UDateTimePatternGenerator *generator;
    enum appcore_time_format timeformat;
    char *timezone_id = NULL;
    UChar utimezone_id[256] = {0,};

    /* reset timezone according to vconf */
    timezone_id = vconf_get_str(VCONFKEY_SETAPPL_TIMEZONE_ID);
    u_uastrcpy(utimezone_id, timezone_id);
    SFREE(timezone_id);
    ucal_setDefaultTimeZone(utimezone_id , &status);
    /* init */
    uloc_setDefault(getenv("LC_TIME"), &status); /* ICU API to set default locale */
    locale = uloc_getDefault();
    /* format */
    appcore_get_timeformat(&timeformat);
    if (timeformat==APPCORE_TIME_FORMAT_24) {
        /* 14 May 09 15:37 */
        u_strFromUTF8(ubuf, MEMO_BUFFER_SIZE, NULL, "dMMMyyHHmm", -1, &status);
    } else {
        /* 14 May 09 03:37 PM */
        u_strFromUTF8(ubuf, MEMO_BUFFER_SIZE, NULL, "dMMMyyhhmma", -1, &status);
    }
    /* get pattern */
    generator = udatpg_open(locale, &status);
    udatpg_getBestPattern(generator, ubuf, u_strlen(ubuf), bestPattern, ICU_TIME_BUFFER, &status);
    udatpg_close(generator);
    /* format */
    if (dfmt != NULL) {
        udat_close(dfmt);
    }
    dfmt = udat_open(UDAT_IGNORE, UDAT_IGNORE, locale, NULL, -1, bestPattern, -1, &status);
}

/**
 * memo_time_format
 *
 * @brief
 *
 * @param   [in] buf    buffer used to store formated time
 *
 * @param   [in] len    length of buf
 *
 * @param   [in] time  actual time
 *
 * @return
 *
 * @exception    None
 *
 * @remark       None
 *
 * @see
 *
 */
void memo_time_format(char *buf, int len, time_t time)
{
    UErrorCode status = U_ZERO_ERROR;
    UChar ubuf[ICU_TIME_BUFFER];

    if (dfmt == NULL) {
        icu_init();
    }
    udat_format(dfmt, (UDate)time*1000, ubuf, ICU_TIME_BUFFER, NULL, &status);
    u_strToUTF8(buf, len, NULL, ubuf, -1, &status);
}

time_t memo_get_binary_release_date()
{
    struct tm time = {0};
    char *date = NULL;
    dictionary *about_dic = iniparser_load("/etc/info.ini");

    RETVIF(about_dic==NULL, 0);
    date = iniparser_getstr(about_dic, "Build:Date");
    strptime(date, "%Y.%m.%d", &time);
    iniparser_freedict(about_dic);
    return timelocal(&time);
}

/**
 * mouse_event_identify
 *
 * @brief
 *
 * @param   [in] startx
 *
 * @param   [in] starty
 *
 * @param   [in] endx
 *
 * @param   [in] endy
 *
 * @return
 *
 * @exception    None
 *
 * @remark       None
 *
 * @see
 *
 */
mouse_behaviour_t mouse_event_identify(int startx, int starty, int endx, int endy)
{
    int gapx = 0;
    int gapy = 0;

    if ((startx == MOUSE_POSITION_INVALID)
        || (starty == MOUSE_POSITION_INVALID)
        || (endx == MOUSE_POSITION_INVALID)
        || (endy == MOUSE_POSITION_INVALID)) {
        return MOUSE_EVENT_INVALID;
    }

    gapx = endx - startx;
    gapy = endy - starty;
    if ((abs(gapx) <= 10) && (abs(gapy) <= 10)) {    /* click */
        return MOUSE_EVENT_CLICK;
    } else if (abs(gapx) > abs(gapy)) {    /* horizonal drag */
        return gapx > 0 ? MOUSE_EVENT_DRAG_RIGHT : MOUSE_EVENT_DRAG_LEFT;
    } else {        /* vertical drag */
        return gapy > 0 ? MOUSE_EVENT_DRAG_DOWN : MOUSE_EVENT_DRAG_UP;
    }
}

static void _evas_object_on_mouse_down(void *data, Evas *e, Evas_Object *evas_obj, void *event_info)
{
    Evas_Event_Mouse_Down *ev = (Evas_Event_Mouse_Down *)event_info;
    evas_object_data_set(evas_obj, "startx", (const void *)ev->output.x);
    evas_object_data_set(evas_obj, "starty", (const void *)ev->output.y);
}

static void _evas_object_on_mouse_up(void *data, Evas *e, Evas_Object *evas_obj, void *event_info)
{
    Evas_Event_Mouse_Down *ev = (Evas_Event_Mouse_Down *)event_info;
    int startx = (int)evas_object_data_get(evas_obj, "startx");
    int starty = (int)evas_object_data_get(evas_obj, "starty");
    Evas_Smart_Cb on_flick = (Evas_Smart_Cb)evas_object_data_get(evas_obj, "on_flick");
    mouse_behaviour_t mbt = mouse_event_identify(startx, starty, ev->output.x, ev->output.y);
    evas_object_data_set(evas_obj, "startx", (const void *)MOUSE_POSITION_INVALID);
    evas_object_data_set(evas_obj, "starty", (const void *)MOUSE_POSITION_INVALID);
    if (on_flick != NULL) {
        on_flick(data, evas_obj, (void *)mbt);
    }
}

void evas_object_flick_callback_add(Evas_Object *obj, Evas_Smart_Cb on_flick, void *data)
{
    evas_object_data_set(obj, "startx", (const void *)MOUSE_POSITION_INVALID);
    evas_object_data_set(obj, "starty", (const void *)MOUSE_POSITION_INVALID);
    evas_object_data_set(obj, "on_flick", (const void *)on_flick);
    evas_object_event_callback_add(obj, EVAS_CALLBACK_MOUSE_DOWN, _evas_object_on_mouse_down, data);
    evas_object_event_callback_add(obj, EVAS_CALLBACK_MOUSE_UP, _evas_object_on_mouse_up, data);
}

void memo_com_dummy_cb(void *data, const char *msg, void *event)
{
    LOGD("msg : %s\n", msg);
}

typedef struct __pop_data_t {
    Evas_Smart_Cb on_yes;
    Evas_Smart_Cb on_no;
    void *user_data;
    Evas_Object *pop;
} pop_data_t;

static void _on_yes_cb(void *data, Evas_Object *obj, void *event_info)
{
	pop_data_t *pData = (pop_data_t *)data;
        if (pData->on_yes != NULL) {
            pData->on_yes(pData->user_data, obj, event_info);
        }
        evas_object_del(pData->pop);
	SFREE(pData);
}

static void _on_no_cb(void *data, Evas_Object *obj, void *event_info)
{
	pop_data_t *pData = (pop_data_t *)data;
        if (pData->on_no != NULL) {
            pData->on_no(pData->user_data, obj, event_info);
        }
        evas_object_del(pData->pop);
	SFREE(pData);
}

/**
 * memo_create_yes_no_popup
 *
 * @brief
 *
 * @param   [in] win
 *
 * @param   [in] msg
 *
 * @param   [in] on_yes
 *
 * @param   [in] on_no
 *
 * @param   [in] data
 *
 * @return
 *
 * @exception    None
 *
 * @remark       None
 *
 * @see
 *
 */
void memo_create_yes_no_popup(Evas_Object *win, const char *msg, Evas_Smart_Cb on_yes, Evas_Smart_Cb on_no, void *data)
{
    pop_data_t *pd = SMALLOC(pop_data_t);
    RETIF(pd == NULL);

    pd->on_yes = on_yes;
    pd->on_no = on_no;
    pd->user_data = data;
    Evas_Object *popup = elm_popup_add(win);
    pd->pop = popup;
    evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_object_text_set(popup, msg);
    Evas_Object *btn1 = elm_button_add(popup);
    elm_object_text_set(btn1, MEMO_I18N_YES);
    elm_object_part_content_set(popup, "button1", btn1);
    evas_object_smart_callback_add(btn1, "clicked", _on_yes_cb, pd);
    Evas_Object *btn2 = elm_button_add(popup);
    elm_object_text_set(btn2, MEMO_I18N_NO);
    elm_object_part_content_set(popup, "button2", btn2);
    evas_object_smart_callback_add(btn2, "clicked", _on_no_cb, pd);
    evas_object_show(popup);
}

void memo_tool_btn_focus_set(Evas_Object *eo, Eina_Bool status)
{
    edje_object_signal_emit(elm_layout_edje_get(eo), (status ? "focus" : "normal"), "elm");
}

static void _set_bg(Evas_Object *obj, service_h service)
{
	unsigned char *color = NULL;
	int bg_color = 0xFFFFFFFF; /* ARGB */
	char *s = NULL;
	service_get_extra_data(service, "bg_color" , &s);
	if (s != NULL) {
		bg_color = atoi(s);
		SFREE(s);
	}
	color = (unsigned char *)&bg_color;
	evas_object_color_set(obj, color[2], color[1], color[0], color[3]);
}

typedef struct __font_size_selector_t {
    Evas_Object *parent;
    Memo_Component_Callback cb;
    void *data;
    /* content */
    Evas_Object *body_main;
    Evas_Object *slider;
    Evas_Object *label;
    Evas_Object *event_rect; /* for sweep */
    /* other */
    int min;
    int max;
    int current;
}font_size_selector_t;

static void _on_font_size_change_cb(void *data, Evas_Object *obj, void *event_info)
{
    font_size_selector_t *fss = (font_size_selector_t *)data;
    fss->current = elm_slider_value_get(fss->slider);
    char buf[MEMO_BUFFER_SIZE];
    snprintf(buf, MEMO_BUFFER_SIZE, "<color=#000000FF><font_size=%d><align=center>Abc</align></font_size></color>", fss->current);
    elm_object_text_set(fss->label, buf);
    fss->cb(fss->data, "change", (void *)fss->current);
}

static void _fss_on_flick(void *data, Evas_Object *evas_obj, void *event_info)
{
    font_size_selector_t *fss = (font_size_selector_t *)data;
    if ((mouse_behaviour_t)event_info == MOUSE_EVENT_DRAG_DOWN) {
        fss->cb(fss->data, "flick,down", (void *)fss->current);
    }
}

void *memo_load_font_size_selector(Evas_Object *parent, service_h service, Memo_Component_Callback cb, void *data)
{
	font_size_selector_t *fss = SMALLOC(font_size_selector_t);
	RETVIF(fss==NULL, NULL);
	service_dump(service);
	fss->parent = parent;
	fss->cb = (cb==NULL ? memo_com_dummy_cb : cb); /* make sure cb is not null, no need to check legitimacy of cb when call */
	fss->data = data;

	/* init */
	fss->min = 20;
	fss->max = 60;
	fss->current = 30;
	char *s = NULL;
	service_get_extra_data(service, "current", &s);
	if (s != NULL) {
		fss->current = atoi(s);
		SFREE(s);
	}

	fss->body_main = elm_layout_create(fss->parent, EDJ_FILE, "font_size_selector");
	fss->cb(fss->data, "layout", fss->body_main);

	/* background */
	Evas_Object *bg = evas_object_rectangle_add(evas_object_evas_get(parent));
	_set_bg(bg, service);
	elm_object_part_content_set(fss->body_main, "bg.color", bg);
	/* slider */
	fss->slider = elm_slider_add(fss->body_main);
	elm_object_style_set(fss->slider, "expanded_indicator_button");
	elm_slider_indicator_show_set(fss->slider, EINA_TRUE);
	elm_slider_min_max_set(fss->slider, fss->min, fss->max);
	elm_slider_indicator_format_set(fss->slider, "%1.0f");
	elm_slider_value_set(fss->slider, fss->current);
	evas_object_smart_callback_add(fss->slider, "changed", _on_font_size_change_cb, fss);
	elm_object_part_content_set(fss->body_main, "elm.swallow.selector", fss->slider);
	/* preview text */
	fss->label = elm_label_add(fss->body_main);
	char buf[MEMO_BUFFER_SIZE];
	snprintf(buf, MEMO_BUFFER_SIZE, "<color=#000000FF><font_size=%d><align=center>Abc</align></font_size></color>", fss->current);
	elm_object_text_set(fss->label, buf);
	elm_object_part_content_set(fss->body_main, "elm.swallow.label", fss->label);
	/* for flick event */
	fss->event_rect = (Evas_Object *)edje_object_part_object_get(elm_layout_edje_get(fss->body_main), "event");
	evas_object_flick_callback_add(fss->event_rect, _fss_on_flick, fss);

	return (void *)fss;
}

void memo_del_font_size_selector(void *h_fss)
{
    font_size_selector_t *fss = (font_size_selector_t *)h_fss;
    evas_object_del(fss->body_main);
    SFREE(fss);
}

typedef struct __color_selector_t {
    Evas_Object *parent;
    Memo_Component_Callback cb;
    void *data;
    /* content */
    Evas_Object *body_main;
    Evas_Object *selector;
    Evas_Object *rect; /* color rect */
    Evas_Object *event_rect; /* for sweep */
    /* other */
    int color; /* ARGB */
}color_selector_t;

static void _on_color_change_cb(void *data, Evas_Object *obj, void *event_info)
{
    int r, g, b, a;
    color_selector_t *cs = (color_selector_t *)data;
    elm_colorselector_color_get(cs->selector, &r, &g, &b, &a);
    evas_object_color_set(cs->rect, r, g, b, a);
    int color = ARGB_JOIN(a, r, g, b);
    if (cs->color != color) {
        cs->color = color;
        cs->cb(cs->data, "change", (void *)cs->color);
    }
}

static void _cs_on_flick(void *data, Evas_Object *evas_obj, void *event_info)
{
    color_selector_t *cs = (color_selector_t *)data;
    if ((mouse_behaviour_t)event_info == MOUSE_EVENT_DRAG_DOWN) {
        cs->cb(cs->data, "flick,down", (void *)cs->color);
    }
}

void *memo_load_color_selector(Evas_Object *parent, service_h service, Memo_Component_Callback cb, void *data)
{
    color_selector_t *cs = SMALLOC(color_selector_t);
    RETVIF(cs==NULL, NULL);
    service_dump(service);
    cs->parent = parent;
    cs->cb = (cb==NULL ? memo_com_dummy_cb : cb); /* make sure cb is not null, no need to check legitimacy of cb when call */
    cs->data = data;

    /* init */
    cs->color = 0xff000000;
    char *s = NULL;
    service_get_extra_data(service, "color" , &s);
    if (s != NULL) {
        cs->color = atoi(s);
	SFREE(s);
    }
    cs->body_main = elm_layout_create(cs->parent, EDJ_FILE, "color_selector");
    cs->cb(cs->data, "layout", cs->body_main);

    /* background */
    service_get_extra_data(service, "bg_color", &s);
    if (s != NULL) {
        edje_object_signal_emit(elm_layout_edje_get(cs->body_main), "white", "elm");
	SFREE(s);
    } else {
        edje_object_signal_emit(elm_layout_edje_get(cs->body_main), "black", "elm");
    }
    /* selector */
    cs->selector = elm_colorselector_add(cs->body_main);
    elm_colorselector_mode_set(cs->selector, ELM_COLORSELECTOR_COMPONENTS);
    elm_object_part_content_set(cs->body_main, "elm.swallow.selector", cs->selector);
    unsigned char *color = (unsigned char *)&cs->color;
    elm_colorselector_color_set(cs->selector, color[2], color[1], color[0], color[3]);
    evas_object_smart_callback_add(cs->selector, "changed", _on_color_change_cb, cs);
    /* preview color rect */
    cs->rect = evas_object_rectangle_add(evas_object_evas_get(parent));
    evas_object_color_set(cs->rect, color[2], color[1], color[0], color[3]);
    elm_object_part_content_set(cs->body_main, "elm.swallow.color", cs->rect);
    /* for flick event */
    cs->event_rect = (Evas_Object *)edje_object_part_object_get(elm_layout_edje_get(cs->body_main), "event");
    evas_object_flick_callback_add(cs->event_rect, _cs_on_flick, cs);

    return (void *)cs;
}

void memo_del_color_selector(void *h_cs)
{
    color_selector_t *cs = (color_selector_t *)h_cs;
    evas_object_del(cs->body_main);
    SFREE(cs);
}

typedef struct __pencil_size_selector_t {
    Evas_Object *parent;
    Memo_Component_Callback cb;
    void *data;
    /* content */
    Evas_Object *body_main;
    Evas_Object *slider;
    Evas_Object *rect; /* previw rect */
    Evas_Object *event_rect; /* for sweep */
    /* other */
    int min;
    int max;
    int current;
}pencil_size_selector_t;

static void _on_pencil_size_change_cb(void *data, Evas_Object *obj, void *event_info)
{
    pencil_size_selector_t *pss = (pencil_size_selector_t *)data;
    pss->current = elm_slider_value_get(pss->slider);
    evas_object_size_hint_min_set(pss->rect, 0, pss->current);
    pss->cb(pss->data, "change", (void *)pss->current);
}

static void _pss_on_flick(void *data, Evas_Object *evas_obj, void *event_info)
{
    pencil_size_selector_t *pss = (pencil_size_selector_t *)data;
    if ((mouse_behaviour_t)event_info == MOUSE_EVENT_DRAG_DOWN) {
        pss->cb(pss->data, "flick,down", (void *)pss->current);
    }
}

void *memo_load_pencil_size_selector(Evas_Object *parent, service_h service, Memo_Component_Callback cb, void *data)
{
    pencil_size_selector_t *pss = SMALLOC(pencil_size_selector_t);
    RETVIF(pss==NULL, NULL);
    service_dump(service);
    pss->parent = parent;
    pss->cb = (cb==NULL ? memo_com_dummy_cb : cb); /* make sure cb is not null, no need to check legitimacy of cb when call */
    pss->data = data;

    /* init */
    pss->min = 1;
    pss->max = 8;
    pss->current = 8;
    char *s = NULL;
    service_get_extra_data(service, "current", &s);
    if (s != NULL) {
        pss->current = atoi(s);
	SFREE(s);
    }

    pss->body_main = elm_layout_create(pss->parent, EDJ_FILE, "pencil_size_selector");
    pss->cb(pss->data, "layout", pss->body_main);

    /* background */
    Evas_Object *bg = evas_object_rectangle_add(evas_object_evas_get(parent));
    _set_bg(bg, service);
    elm_object_part_content_set(pss->body_main, "bg.color", bg);
    /* slider */
    pss->slider = elm_slider_add(pss->body_main);
    elm_object_style_set(pss->slider, "expanded_indicator_button");
    elm_slider_indicator_show_set(pss->slider, EINA_TRUE);
    elm_slider_min_max_set(pss->slider, pss->min, pss->max);
    elm_slider_indicator_format_set(pss->slider, "%1.0f");
    elm_slider_value_set(pss->slider, pss->current);
    evas_object_smart_callback_add(pss->slider, "changed", _on_pencil_size_change_cb, pss);
    elm_object_part_content_set(pss->body_main, "elm.swallow.selector", pss->slider);
    /* preview size rect */
    pss->rect = evas_object_rectangle_add(evas_object_evas_get(parent));
    evas_object_color_set(pss->rect, 0, 0, 0, 255);
    evas_object_size_hint_min_set(pss->rect, 0, pss->current);
    elm_object_part_content_set(pss->body_main, "elm.swallow.pencil", pss->rect);
    /* for flick event */
    pss->event_rect = (Evas_Object *)edje_object_part_object_get(elm_layout_edje_get(pss->body_main), "event");
    evas_object_flick_callback_add(pss->event_rect, _pss_on_flick, pss);

    return (void *)pss;
}

void memo_del_pencil_size_selector(void *h_pss)
{
    pencil_size_selector_t *pss = (pencil_size_selector_t *)h_pss;
    evas_object_del(pss->body_main);
    SFREE(pss);
}

void memo_navigator_pop(Evas_Object *navi)
{
    Elm_Object_Item *top_it = elm_naviframe_top_item_get(navi);
    Evas_Object *eo = elm_object_item_content_get(top_it);
    Evas_Object *prev_eo = evas_object_data_get(eo, "previous_obj");
    if(prev_eo == NULL) { /* exit when view stack is empty */
        elm_exit();
    } else {
        Evas_Smart_Cb cb = evas_object_data_get(prev_eo, "update_cb");
        RETIF(cb == NULL);
        void *cb_data = evas_object_data_get(prev_eo, "update_data");
        cb(cb_data, NULL, NULL);
    }
    elm_naviframe_item_pop(navi);
}

Elm_Object_Item *memo_naviframe_item_push(Evas_Object *obj, const char *title_label,
    Evas_Object *prev_btn, Evas_Object *next_btn, Evas_Object *content, const char *item_style)
{
    Elm_Object_Item *top_it = elm_naviframe_top_item_get(obj);
    if (top_it != NULL) {
        Evas_Object *eo = elm_object_item_content_get(top_it);
        if (eo != NULL) {
            evas_object_data_set(content, "previous_obj", eo);
        }
    }

    return elm_naviframe_item_push(obj, title_label, prev_btn, next_btn, content, item_style);
}

Evas_Object *memo_controlbar_add(Evas_Object *parent)
{
	Evas_Object *toolbar = elm_toolbar_add(parent);
	elm_toolbar_shrink_mode_set(toolbar, ELM_TOOLBAR_SHRINK_EXPAND);
	elm_toolbar_homogeneous_set(toolbar, EINA_FALSE);
	return toolbar;
}

void memo_comment_recover(Evas_Object *entry, const char *orig_utf8)
{
    if (orig_utf8 == NULL) {
        elm_entry_entry_set(entry, MEMO_I18N_ADD_COMMENT);
    } else {
        char *content = elm_entry_utf8_to_markup(orig_utf8);
        elm_entry_entry_set(entry, content);
        SFREE(content);
    }
}
