/*
*
* Copyright 2012  Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
*/

#include <Elementary.h>
#include <dlog.h>
#include <gravel.h>
#include <extended-elm.h>
#include <assert.h>
#include <glib.h>
#include <memo_string.h>
#include <aul.h>
#include <appsvc.h>
#include <contacts-ug.h>
#include <memo_autolink.h>
#include <memo_ug.h>

static void _autolink_popup_response_cb(void *data, Evas_Object *obj, void *event_info);
/* BEGIN CONVERTE BETWEEN enum anchor_t AND string  */

static char *_autolink_anchor_type_to_string(enum anchor_t type)
{
    switch (type) {
    case ANCHOR_EMAIL:
        return "email";
    case ANCHOR_PHONE:
        return "phone_num";
    case ANCHOR_URL:
        return "url";
    default:
        return "";
    }
}

/* END CONVERTE BETWEEN enum anchor_t AND string  */

/* BEGIN POPUP CALLBACK */

static void _autolink_send_email_selected_cb(void *data, Evas_Object *obj, void *event_info)
{
	struct autolink_data_t *ald = (struct autolink_data_t *)data;
	service_h service = NULL;
	service_create(&service);
	service_add_extra_data(service, "RUN_TYPE", "5");
	service_add_extra_data(service, "TO", ald->info);

	ug_launch_common(service, UG_NAME_EMAIL);
	_autolink_popup_response_cb(ald, ald->popup, NULL);
}

static void _autolink_send_message_selected_cb(void *data, Evas_Object *obj, void *event_info)
{
    struct autolink_data_t *ald = (struct autolink_data_t *)data;

    ug_launch_common_var(UG_NAME_MESSAGE, "TO", ald->info, NULL);
    _autolink_popup_response_cb(ald, ald->popup, NULL);
}

static void _autolink_vioce_call_selected_cb(void *data, Evas_Object *obj, void *event_info)
{
	struct autolink_data_t *ald = (struct autolink_data_t *)data;
	bundle *bd = bundle_create();
	char telnum[255] = {0,};

	appsvc_set_operation(bd, APPSVC_OPERATION_CALL);
	snprintf(telnum, sizeof(telnum), "tel:%s", ald->info);
	appsvc_set_uri(bd, telnum);
	appsvc_run_service(bd, 0, NULL, NULL);

	bundle_free(bd);
	_autolink_popup_response_cb(ald, ald->popup, NULL);
}

static void _autolink_video_call_selected_cb(void *data, Evas_Object *obj, void *event_info)
{
	struct autolink_data_t *ald = (struct autolink_data_t *)data;
	service_h service = NULL;
	service_create(&service);
	service_add_extra_data(service, "KEY_CALL_TYPE", "mo");
	service_add_extra_data(service, "KEY_CALL_HANDLE", "1");
	service_add_extra_data(service, "KEY_CALLING_PARTY_NUMBER", ald->info);
	service_add_extra_data(service, "KEY_CLI_CAUSE", "-1");
	service_add_extra_data(service, "KEY_FORWARDED", "-1");
	service_set_package(service, AUL_NAME_VEDIO_CALL);
	service_send_launch_request(service, NULL, NULL);

	service_destroy(service);
	_autolink_popup_response_cb(ald, ald->popup, NULL);
}

static void _autolink_email_add_to_contact_selected_cb(void *data, Evas_Object *obj,
                               void *event_info)
{
	struct autolink_data_t *ald = (struct autolink_data_t *)data;

	char buf[10];
	snprintf(buf, sizeof(buf), "%d", CT_UG_REQUEST_ADD_WITH_EMAIL);

	service_h service = NULL;
	service_create(&service);
	service_add_extra_data(service, CT_UG_BUNDLE_TYPE, buf);
	service_add_extra_data(service, CT_UG_BUNDLE_EMAIL, ald->info);

	ug_launch_common(service, UG_CONTACTS_DETAILS);
	_autolink_popup_response_cb(ald, ald->popup, NULL);
}

static void _autolink_phone_add_to_contact_selected_cb(void *data, Evas_Object *obj,
                               void *event_info)
{
	struct autolink_data_t *ald = (struct autolink_data_t *)data;

	char buf[10];
	snprintf(buf, sizeof(buf), "%d", CT_UG_REQUEST_ADD_WITH_NUM);

	service_h service = NULL;
	service_create(&service);
	service_add_extra_data(service, CT_UG_BUNDLE_TYPE, buf);
	service_add_extra_data(service, CT_UG_BUNDLE_NUM, ald->info);

	ug_launch_common(service, UG_CONTACTS_DETAILS);
	_autolink_popup_response_cb(ald, ald->popup, NULL);
}

/* END POPUP CALLBACK */

static struct anchor_popup_item_t g_email_list[] = {
    {"IDS_MEMO_OPT_SEND_EMAIL", _autolink_send_email_selected_cb, "memo"},
    {"IDS_MEMO_BODY_ADD_TO_CONTACT", _autolink_email_add_to_contact_selected_cb, "memo"},
};

static struct anchor_popup_item_t g_phone_list[] = {
    {"IDS_COM_BODY_VOICE_CALL", _autolink_vioce_call_selected_cb, "sys_string"},
    {"IDS_COM_BODY_SEND_MESSAGE", _autolink_send_message_selected_cb, "sys_string"},
    {"IDS_COM_BODY_VIDEO_CALL", _autolink_video_call_selected_cb, "sys_string"},
    {"IDS_MEMO_BODY_ADD_TO_CONTACT", _autolink_phone_add_to_contact_selected_cb, "memo"},
};

static char *_autolink_gl_label_get(void *data, Evas_Object *obj, const char *part)
{
    char *label = (char *)data;
    if (strcmp(part, "elm.text") == 0) {
        return strdup(label);
    }
    return NULL;
}

static void _autolink_popup_response_cb(void *data, Evas_Object *obj, void *event_info)
{
    PFUNC_ENTER;

    assert(data != NULL);
    struct autolink_data_t *ald = (struct autolink_data_t *)data;
    if (ald->popup != NULL) {
        evas_object_del(ald->popup);
    }

    assert(ald->info != NULL);
    free(ald->info);
    free(ald);

    PFUNC_LEAVE;
}

void _autolink_popup_show(char *title, struct autolink_data_t *ald)
{
    struct anchor_popup_item_t *list = NULL;
    int count = 0;
    Evas_Object *box = NULL;

    switch (ald->type) {
    case ANCHOR_EMAIL:
        list = g_email_list;
        count = ARRAY_SIZE(g_email_list);
        break;
    case ANCHOR_PHONE:
        list = g_phone_list;
        count = ARRAY_SIZE(g_phone_list);
        break;
    default:
        break;
    }

    Evas_Object *popup = elm_popup_add(ald->win_main);
    ald->popup = popup;
    elm_object_part_text_set(popup, "title,text", title);
    Evas_Object *btn1 = elm_button_add(popup);
    elm_object_text_set(btn1, MEMO_I18N_CLOSE);
    elm_object_part_content_set(popup, "button1", btn1);
    evas_object_smart_callback_add(btn1, "clicked", _autolink_popup_response_cb, ald);
    evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

    static Elm_Genlist_Item_Class itc;
    memset(&itc, 0, sizeof(Elm_Genlist_Item_Class));
    itc.item_style = "1text";
    itc.func.text_get = _autolink_gl_label_get;

    Evas_Object *genlist = elm_genlist_add(popup);
    evas_object_smart_callback_add(genlist, "selected", NULL, NULL);
    evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);

    int i = 0;
    for (i = 0; i < count; ++i) {
        elm_genlist_item_append(genlist, &itc, dgettext(list[i].domain, list[i].label),
                    NULL, ELM_GENLIST_ITEM_NONE, list[i].response, ald);
    }

    evas_object_show(genlist);
    box = elm_box_add(popup);
    evas_object_size_hint_min_set(box, 0, (count>3 ? 3 : count)*71);
    elm_box_pack_end(box, genlist);
    evas_object_show(box);
    elm_object_content_set(popup, box);
    evas_object_show(popup);
}

void autolink_anchor_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
    Elm_Entry_Anchor_Info *ei = (Elm_Entry_Anchor_Info *)event_info;
    autolink_data *ald = SMALLOC(autolink_data);
    RETIF(ald == NULL);

    int info_len = strlen(ei->name) + 1;
    ald->info = calloc(1, info_len);
    if (ald->info == NULL) {
        SFREE(ald);
        return;
    }

    ald->win_main = (Evas_Object *)data;
    /* retrieve anchor info, eg `url|www  phone_num|8888 email|abc@k.com` */
    switch (ei->name[0]) {
    case 'u':
        ald->type = ANCHOR_URL;
        break;
    case 'p':
        ald->type = ANCHOR_PHONE;
        break;
    case 'e':
        ald->type = ANCHOR_EMAIL;
        break;
    default :
        ald->type = NOT_ANCHOR;
        break;
    }
    const char *start = ei->name + strlen(_autolink_anchor_type_to_string(ald->type)) +1;
    const char *end = strchr(ei->name, ' ');
    strncpy(ald->info, start, end - start);
    LOGD("Anchor clicked (%s)\n", ald->info);

    if ((ald->type == ANCHOR_EMAIL) || (ald->type == ANCHOR_PHONE)){
        _autolink_popup_show(ald->info, ald);
        return;
    }

    if (ald->type == ANCHOR_URL) {
        aul_open_content(ald->info);    /* launch browser */
    }
    SFREE(ald->info);
    SFREE(ald);
}

static Eina_Bool str_is_begin_with(const char *str, const char *prefix)
{
    if (strlen(str) < strlen(prefix)) {
        return EINA_FALSE;
    }

    int i = 0;
    for (i = 0; i < strlen(prefix); ++i) {
        if (str[i] != prefix[i]) {
            return EINA_FALSE;
        }
    }

    return EINA_TRUE;
}

/* TYPE CHECKER BEGIN */
/*
 * [Checker Function Specification]
 * These function get target type from beginning of string.
 * The only parameter str is the string that want to be checked.
 * Let's assume the returned value is r. There are 3 cases:
 *     r > 0 : From beginning to r of str match the target type.
 *     r = 0 : Str doesn't have target type, no longer check.
 *     r < 0 : It doesn't match from beginning, but from -r position maybe matched.
 *             It will be checked next time.
 */
static int _autolink_url_checker(const char* str)
{
    char *prefix[] = {"http://", "www.", "wap."};

    int i = 0;
    for (i = 0; i < ARRAY_SIZE(prefix); ++i) {
        if (str_is_begin_with(str, prefix[i])) { /* Begin with specified prefix */
            int j = 0;
            while(str[j] != '\0' && !isspace(str[j])) {
                if (str[j] == '<') { /* tag begin */
                    break;
                }
                j++;
            }

            if(j == strlen(prefix[i])) { /* only with prefix alone */
                break;
            } else {
                return j;
            }
        }
    }

    /* not matched, get next position */
#if 0 /* Need correct */
    int next = 0;
    char *p = NULL;
    for (i = 0; i < ARRAY_SIZE(prefix); ++i) {
        p = strstr(str, prefix[i]);
        if (p != NULL) {
            if (next == 0) {
                next = p - str;
            } else if (next > p - str) {
                next = p - str;
            }
        }
    }
#endif

    return -1;
}

static int _autolink_email_checker(const char* str)
{
    char *p = NULL;
    p = strchr(str, '@');

    if (NULL == p) {
        return 0;
    } else if (p == str) { /* '@' at beginning */
        return -1;
    }

    char next = p[1];
    if (next == '@') { /* consecutive '@' */
        return -(p + 2 - str);
    } else if (next == '.') { /* "@." */
        return -(p + 2 - str);
    } else if (next == '\0') { /* '@' at end of string */
        return -(p + 1 - str);
    } else if (isspace(next) || next == '<') { /* '@' at end of word */
        return -(p + 2 - str);
    }

    /* check if valid before @ */
    char *q = p;
    while (q >= str) {
        if (isspace(*q)) {
            return -(q + 1 - str);
        }

        if (*q == '>' || *q == '/') { /* tag end and other illega character */
            return -(q + 1 - str);
        }

        if (*q == '.' && ( q == str || *(q - 1) == '.')) {
            return -(q + 1 - str);
        }

        q--;
    }

    /* get the end position of email */
    q = p + 1;
    while (*q != '\0' && !isspace(*q)) {
        if (*q == '.' && *(q + 1) == '.') {
            break;
        }

        if (*q == '<') { /* tag begin */
            break;
        }

        q++;
    }
    return q - str;
}

static int _autolink_phone_checker(const char *str)
{
    /* search beginning */
    int i = 0;
    for (i = 0; str[i] != '\0'; ++i) {
        /* digit, '+', '*' and '#' at the beginning of phone is valid. */
        if (isdigit(str[i]) || str[i] == '+' || str[i] == '*' || str[i] == '#') {
            break;
        }
    }

    if (i > 0) {
        return -i;
    }

    for (i = 1; str[i] != '\0'; ++i) {
        if (isdigit(str[i]) || str[i] == '*' || str[i] == '#') {
            /* NOP */
        } else if (str[i] == '-') {
            if (str[i - 1] == '-' || str[i - 1] == ' ') { /* "--" or " -" */
                i -= 1;
                break;
            }
        } else if (str[i] == ' ') {
            if (str[i - 1] == '-') { /* "- " */
                i -= 1;
                break;
            } else if (str[i - 1] == ' ' && isdigit(str[i + 1])) { /* consecutive ' ' */
                int j = 0;
                while (isdigit(str[i + 1 + j])) { ++j; }
                if (j > 5) {
                    while (str[i] == ' ') { i--; }
                    return i + 1;
                }
            }
        } else { /* other character */
            break;
        }
    }

    int len = i;
    if (len < 3) {
        return -len;
    } else {
        return len;
    }
}

/* TYEP CHECKER END */

static void _autolink_box_anchor(GString *text, char *type, char *content)
{
    char *buf = NULL;
    if (strcmp(type, "phone_num") == 0) { /* remove '-' and ' ' in phone num. */
        buf = (char *)malloc(strlen(content) + 1);
        RETIF(buf == NULL);
        int i = 0, j = 0;
        while (content[i] != '\0') {
            if (content[i] == '-' || content[i] == ' ') {
                ++i;
            } else {
                buf[j] = content[i];
                ++i;
                ++j;
            }
        }
        buf[j] = '\0';
    } else {
        buf = content;
    }

    g_string_append_printf(text,
        "<color=#72b1f2FF><a href=%s|%s underline=on underline_color=#72b1f2FF>%s</a></color>",
        type, buf, content);

    if (buf != content) {
        free(buf);
    }
}

GString *autolink_add_anchor(const char *content)
{
    assert(content != NULL);

    const char *p = content;
    char *gt = NULL;
    GString *text = g_string_new("");
    GString *buf = g_string_new("");

    struct checker_t {
        int (*check)(const char* str);
        char *type;
        Eina_Bool invoke;
    } checkers[] = {
        {_autolink_url_checker, "url", EINA_TRUE},
        {_autolink_email_checker, "email", EINA_TRUE},
        {_autolink_phone_checker, "phone_num", EINA_TRUE},
    };

    while (*p != '\0') {

        if (p[0] == '<') {    /* tag begin */
            gt = strchr(p, '>');    /* search tag end */
            assert(gt != NULL);    /* assert '<' and '>' are matched */
            g_string_append_len(text, p, gt - p + 1);
            p = gt + 1;
        }

        int i = 0;
        int r = 0;
        int next_pos = -1;
        for (i = 0; i < ARRAY_SIZE(checkers); ++i) {
            if (checkers[i].invoke == EINA_TRUE) {
                r = checkers[i].check(p);
                if (r == 0) { /* no longer check */
                    checkers[i].invoke = EINA_FALSE;
                } else if (r > 0) { /* Get */
                    g_string_assign(buf, "");    /* reset buffer */
                    g_string_append_len(buf, p, r);
                    _autolink_box_anchor(text, checkers[i].type, buf->str);
                    next_pos = r;
                    break;
                } else { /* r < 0 */
                    if (next_pos <= 0 || next_pos > -r) {
                        next_pos = -r;
                    }
                }
            }
        }

        if (next_pos < 0) {
            g_string_append(text, p);
            break;
        } else if ( i == ARRAY_SIZE(checkers)) {
            g_string_append_len(text, p, next_pos);
        }

        p += next_pos;
    }

    g_string_free(buf, TRUE);
    printf("\t\t********** %s\n", text->str);

    return text;
}
