/*
*
* Copyright 2012  Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
*/
#include <ui-gadget.h>
#include <Ecore_X.h>
#include <aul.h>
#include <supplement.h>
#include <memo-db.h>
#include "memo_ug.h"

static void _ug_layout_cb(ui_gadget_h ug, enum ug_mode mode, void *priv)
{
    Evas_Object *base, *win;
    base = (Evas_Object *)ug_get_layout(ug);
    if (base == NULL) {
        return;
    }

    win = ug_get_window();
    switch (mode) {
    case UG_MODE_FULLVIEW:
        evas_object_size_hint_weight_set(base, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        elm_win_resize_object_add(win, base);
        /* disable effect */
        //ug_disable_effect(ug);
        evas_object_show(base);
        break;
    default:
        break;
    }
}

static void _ug_result_cb(ui_gadget_h ug, service_h service, void *priv)
{
    service_dump(service);
}

static void _ug_destroy_cb(ui_gadget_h ug, void *priv)
{
    if (ug != NULL) {
        ug_destroy(ug);
    }
}

/**
 * ug_launch_common
 *
 * @description
 *  This is a basic function which designed to invoke any ui gadget.
 *
 * @param[in]   service         a service type which will send to ug directly
 * @param[in]   ug_name    UG name marco defined in memo_ug.h
 * @return      void
 */
void ug_launch_common(service_h service, char *ug_name)
{
    ug_cbs_t cbs={0, };

    cbs.layout_cb = _ug_layout_cb;
    cbs.destroy_cb = _ug_destroy_cb;
    cbs.result_cb = _ug_result_cb;
    cbs.priv = NULL;

    ug_create(NULL, ug_name, UG_MODE_FULLVIEW, service, &cbs);
    service_destroy(service);
}

/**
 * ug_launch_common_var
 *
 * @description
 *  This function supply a variadic version of ug_launch_common_var.
 *  The first and only required variable is the ug_name.
 *  Its type is string, but the caller should use the macro name which defined in memo_ug.h
 *  The other variables are optional, it should appear for key-value pair
 *  The end of optional variables must be NULL.
 *
 * @param[in]   ug_name    UG name marco defined in memo_ug.h
 * @param[in]   ...        pair of key and value, end with NULL
 * @return      void
 */
void ug_launch_common_var(char *ug_name, ...)
{
	char *key = NULL;
	char *val = NULL;
	service_h service = NULL;
	service_create(&service);

	va_list ap;
	va_start(ap, ug_name);
	while (1) {
		key = va_arg(ap, char *);
		val = va_arg(ap, char *);

		if (key == NULL || val == NULL) {
			break;
		}
		service_add_extra_data(service, key, val);
	}
	va_end(ap);

	ug_launch_common(service, ug_name);
}

