/*
*
* Copyright 2012  Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
*/

#include <stdlib.h>
#include <stdio.h>
#include <Elementary.h>
#include <dlog.h>
#include <ui-gadget-module.h>
#include <gravel.h>
#include <memo_string.h>
#include <extended-elm.h>
#include <supplement.h>
#include <memo-assist.h>
#include <memo-genlist.h>
#include <memo-assist.h>
#include <memo_ug.h>
#include <memo_autolink.h>
#include <memo-efl.h>

typedef struct __detail_view_t {
    ug_data_t *ugd;
    Evas_Object *navigator;
    Evas_Object *ctrl_bar;
    Evas_Object *body_main;
    Evas_Object *entry;
    char *content;

    /* add more variables here */
    int index;
	memo_data_t * md;
} detail_view_t;

static void _on_quit(void *data, Evas_Object *obj, void *event_info)
{
    RETIF(data == NULL);
    detail_view_t *dv = (detail_view_t *)data;
    ug_data_t *ugd = dv->ugd;
    ug_destroy_me(ugd->ug);
}

static void _on_entry_resize(void *data, Evas_Object *obj, void *event_info)
{
    LOGD("---------------_on_entry_resize");
    detail_view_t *dv = (detail_view_t *)data;
    elm_entry_entry_set(dv->entry, dv->content);
    SFREE(dv->content);
}

static void _create_text_viewer_layout(detail_view_t *dv)
{
	dv->body_main = elm_layout_create(dv->navigator, EDJ_FILE, "detail_frame");
    /* date */
    memo_time_format(dv->ugd->buf, MEMO_BUFFER_SIZE, dv->md->modi_time);
    edje_object_part_text_set(_EDJ(dv->body_main), "elm.text.date", dv->ugd->buf);
    /* content */
	#if 0
	char buf[MEMO_BUFFER_SIZE];
    GString *text =
        autolink_add_anchor(elm_entry_utf8_to_markup
                (md->content));
    unsigned char *color = (unsigned char *)&md->font_color;
    snprintf(buf, MEMO_BUFFER_SIZE,
        "<font_size=%d><color=#%02x%02x%02x>",
        md->font_size, color[2], color[1], color[0]);
    g_string_prepend(text, buf);
	g_string_free(text, TRUE);
	#endif

	char * markup = elm_entry_utf8_to_markup
                (dv->md->content);

    Evas_Object *sc = NULL;
    sc = elm_swallowed_scroller(dv->body_main, "elm.swallow.content");
    Evas_Object * content = elm_entry_create(dv->body_main, markup);
    dv->content = strdup(markup);
	SFREE(markup);
    //evas_object_smart_callback_add(content, "anchor,clicked", autolink_anchor_clicked_cb, ug_get_parent_layout(ugd->ug));
    evas_object_size_hint_weight_set(content, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(content, EVAS_HINT_FILL, EVAS_HINT_EXPAND);
    elm_entry_editable_set(content, EINA_FALSE);
    elm_entry_autocapital_type_set(content, ELM_AUTOCAPITAL_TYPE_SENTENCE);
    elm_object_content_set(sc, content);
    dv->entry = content;
    evas_object_render_flush_hook(dv->entry, _on_entry_resize, dv);

    edje_object_signal_emit(_EDJ(dv->body_main), "sig_text_mode", "");
}

static void _create_doodle_viewer_layout(detail_view_t *dv)
{
	dv->body_main = elm_layout_create(dv->navigator, EDJ_FILE, "doodle_viewer");

	/* date */
	memo_time_format(dv->ugd->buf, MEMO_BUFFER_SIZE, dv->md->modi_time);
	edje_object_part_text_set(elm_layout_edje_get(dv->body_main), "elm.text.date", dv->ugd->buf);

	/* doodle */
	snprintf(dv->ugd->buf, MEMO_BUFFER_SIZE, DOODLEDIR "/%d.png", dv->md->id);
	Evas_Object * doodle = elm_swallowed_icon(dv->body_main, "elm.swallow.doodle", dv->ugd->buf);

	/* comment */
	Evas_Object *eo = elm_swallowed_layout(dv->body_main, "elm.swallow.comment", EDJ_FILE, "doodle_comment");
	Evas_Object *ef = elm_entry_add(eo);
	elm_object_part_content_set(eo, "elm.swallow.editfield", ef);

	elm_entry_editable_set(ef, EINA_FALSE);
	elm_entry_single_line_set(ef, EINA_TRUE);
	elm_entry_scrollable_set(ef, EINA_TRUE);
	dv->entry = ef;
	elm_entry_cnp_mode_set(dv->entry, ELM_CNP_MODE_PLAINTEXT);
	memo_comment_recover(dv->entry, dv->md->comment);
}

static void _create_detail_layout(detail_view_t *dv)
{
    dv->md = memo_get_data(dv->index);

    /* add navigationbar */
    dv->navigator = elm_naviframe_add(dv->ugd->base);
    if (dv->md != NULL) {
        if(!dv->md->has_doodle) {
			_create_text_viewer_layout(dv);
        } else {
			_create_doodle_viewer_layout(dv);
		}
        memo_free_data(dv->md);
    }

	/* prev button */
	Evas_Object *btn = elm_button_add(dv->navigator);
	evas_object_smart_callback_add(btn, "clicked", _on_quit, dv);
	elm_object_style_set(btn, "naviframe/back_btn/default");
	/* compose */
	Elm_Object_Item *navi_it = memo_naviframe_item_push(dv->navigator, MEMO_I18N_MEMO, btn, NULL, dv->body_main, NULL);
}

void *memo_load_detail_view(ug_data_t *ugd, service_h service)
{
    detail_view_t *dv = SMALLOC(detail_view_t);
    RETVIF(dv == NULL, NULL);
    dv->ugd = ugd;

    /* init */
    char *str = NULL;;
    service_get_extra_data(service, "index", &str);
	LOGD("index = [%s]",str);
    if (str != NULL) {
        dv->index = atoi(str);
	SFREE(str);
    } else {
        SFREE(dv);
        return NULL;
    }

    /* create */
    _create_detail_layout(dv);
    elm_object_part_content_set(ugd->base, "elm.swallow.content", dv->navigator);
    return (void *)dv;
}
