/*
*
* Copyright 2012  Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
*/

#include <stdlib.h>
#include <stdio.h>
#include <Elementary.h>
#include <dlog.h>
#include <ui-gadget-module.h>
#include <gravel.h>
#include <memo_string.h>
#include <extended-elm.h>
#include <supplement.h>
#include <memo-assist.h>
#include <memo-genlist.h>
#include <memo-assist.h>
#include <memo-efl.h>

typedef struct __select_view_t {
    ug_data_t *ugd;
    Evas_Object *navigator;
    Evas_Object *ctrl_bar;
    Evas_Object *r_btn;
    Evas_Object *body_main;

    /* add more variables here */
    Elm_Genlist_Item_Class itc;
    Elm_Genlist_Item_Class itc_draw;
    Evas_Object *genlist;
    int selected;
    int total;
    Evas_Object *selectioninfo;
    Evas_Object *selectioninfo_layout;

    Eina_Bool is_textonly;
} select_view_t;

static void memo_data_iterate_cb (memo_data_t *md, void *user_data);

static void _on_data_update(void *data)
{
    select_view_t *sv = (select_view_t *)data;
    elm_genlist_clear(sv->genlist);
        memo_all_data(memo_data_iterate_cb, sv);
    sv->selected = 0;
        elm_object_disabled_set(sv->r_btn, EINA_TRUE);
}

static void _on_quit(void *data, Evas_Object *obj, void *event_info)
{
    RETIF(data == NULL);
    select_view_t *sv = (select_view_t *)data;
    ug_data_t *ugd = sv->ugd;
    memo_unsubscribe_change(_on_data_update);
    ug_destroy_me(ugd->ug);
}

static void _on_add(void *data, Evas_Object *obj, void *event_info)
{
    RETIF(data == NULL);
    select_view_t *sv = (select_view_t *)data;
    ug_data_t *ugd = sv->ugd;
    service_h service = NULL;
    Elm_Object_Item *it;
    gl_data_t *gld = NULL;
    int c_text = 0;
    int c_image = 0;
    char path[MEMO_BUFFER_SIZE] = {0};
    memo_data_t *md = NULL;

    service_create(&service);
    it = elm_genlist_first_item_get(sv->genlist);
    while (it != NULL) {
        gld = (gl_data_t *)elm_object_item_data_get(it);
        md = memo_get_data(gld->index);
        if (gld->check) {
            if (gld->has_doodle) {
                c_image++;
                snprintf(ugd->buf, MEMO_BUFFER_SIZE, "image%d", c_image);
                snprintf(path, MEMO_BUFFER_SIZE, "/opt/apps/org.tizen.memo/data/doodle/%d.png", gld->index);
                service_add_extra_data(service, ugd->buf, path);
				LOGD("path = (%s)", path);
            } else {
                c_text++;
				int length = strlen(md->content) + 2;
				char * text = (char *) calloc(1, length);
                snprintf(ugd->buf, MEMO_BUFFER_SIZE, "text%d", c_text);
				memcpy(text, md->content, length - 2);
				text[length - 2] = '\n';
				text[length - 1] = '\0';
                service_add_extra_data(service, ugd->buf, text);
				if(text) {
					free(text);
					text = NULL;
				}
            }
        }
        memo_free_data(md);
        it = elm_genlist_item_next_get(it);
    }

    snprintf(ugd->buf, MEMO_BUFFER_SIZE, "%d", c_image);
    service_add_extra_data(service, "image", ugd->buf);
    snprintf(ugd->buf, MEMO_BUFFER_SIZE, "%d", c_text);
    service_add_extra_data(service, "text", ugd->buf);
    ug_send_result(ugd->ug, service);
    service_dump(service);
    service_destroy(service);
    memo_unsubscribe_change(_on_data_update);
    ug_destroy_me(ugd->ug);
}

#pragma GCC diagnostic ignored "-Wformat-extra-args"
static void _create_selectioninfo(select_view_t *sv)
{
    if (sv->selectioninfo == NULL)
    {
        sv->selectioninfo = elm_notify_add(sv->body_main);
        elm_notify_orient_set(sv->selectioninfo, ELM_NOTIFY_ORIENT_BOTTOM);
        sv->selectioninfo_layout = elm_layout_add(sv->body_main);
        elm_layout_theme_set(sv->selectioninfo_layout, "standard", "selectioninfo", "vertical/bottom_12");
        elm_object_content_set(sv->selectioninfo, sv->selectioninfo_layout);
    }

    if (sv->selected == 0) {
        evas_object_hide(sv->selectioninfo);
    } else {
        char text[128];
        if (sv->selected == 1) {
            snprintf(text, 128, "%s", MEMO_I18N_1_MEMO_SELECTED);
        } else {
            snprintf(text, 128, MEMO_I18N_D_MEMOS_SELECTED, sv->selected);
        }
        edje_object_part_text_set(_EDJ(sv->selectioninfo_layout), "elm.text", text);
        evas_object_show(sv->selectioninfo);
    }
}
#pragma GCC diagnostic warning "-Wformat-extra-args"

static void _on_item_select_cb(void *data, Evas_Object *obj, void *event_info)
{
    Elm_Object_Item *it = (Elm_Object_Item *)event_info;
    gl_data_t *gld = (gl_data_t *)elm_object_item_data_get(it);
    select_view_t *sv = (select_view_t *)gld->user_data;

    gld->check = !gld->check;
    elm_genlist_item_update(it);
    gld->check ? sv->selected++ : sv->selected--;

    /* update add button and selectinfo */
    if (sv->selected == 0) {
        elm_object_disabled_set(sv->r_btn, EINA_TRUE);
    } else {
        elm_object_disabled_set(sv->r_btn, EINA_FALSE);
    }
    _create_selectioninfo(sv);

    /* reset state, next click event will be triggered as selected */
    elm_genlist_item_selected_set(it, 0);
}

static void memo_data_iterate_cb (memo_data_t *md, void *user_data)
{
    select_view_t *sv = (select_view_t *)user_data;
    gl_data_t *gld = SMALLOC(gl_data_t);
    RETIF(gld == NULL);
    memo_gld_init(gld, md, sv, NULL);
    if (md->has_doodle) {
        if (!sv->is_textonly) {
            elm_genlist_item_append(sv->genlist, &sv->itc_draw,(void *)gld,
                NULL, ELM_GENLIST_ITEM_NONE, _on_item_select_cb, NULL);
            sv->total++;
        } else {
		SFREE(gld);
        }
    } else {
        elm_genlist_item_append(sv->genlist, &sv->itc,(void *)gld,
            NULL, ELM_GENLIST_ITEM_NONE, _on_item_select_cb, NULL);
        sv->total++;
    }
}

static void _create_select_layout(select_view_t *sv)
{
    ug_data_t *ugd = sv->ugd;

    /* add navigationbar */
    sv->navigator = elm_naviframe_add(ugd->base);

    /* body main */
    sv->body_main = elm_layout_create(sv->navigator, EDJ_FILE, "selector_frame");
    /* genlist */
    sv->genlist = elm_genlist_add(sv->body_main);
    /* apply customized theme */
    elm_object_theme_set(sv->genlist, sv->ugd->th);
    elm_genlist_block_count_set(sv->genlist, 10);
    memo_all_data(memo_data_iterate_cb, sv);

    elm_object_part_content_set(sv->body_main, "elm.swallow.content", sv->genlist);
    edje_object_part_text_set(_EDJ(sv->body_main), "text", (sv->total == 0 ? "No item" : ""));

    /* compose */
    Elm_Object_Item *navi_it = elm_naviframe_item_push(sv->navigator, MEMO_I18N_MEMO, NULL, NULL, sv->body_main, NULL);
    /* control bar */
	Evas_Object *btn1 = elm_button_add(sv->navigator);
	elm_object_text_set(btn1, MEMO_I18N_ADD);
	elm_object_style_set(btn1, "naviframe/toolbar/default");
	elm_object_item_part_content_set(navi_it, "toolbar_button1", btn1);
	evas_object_smart_callback_add(btn1, "clicked", _on_add,
								sv);
	sv->r_btn = btn1;
	elm_object_disabled_set(sv->r_btn, EINA_TRUE); /* disable add defaultly */

	Evas_Object *btn2 = elm_button_add(sv->navigator);
	elm_object_text_set(btn2, MEMO_I18N_CANCEL);
	elm_object_style_set(btn2, "naviframe/toolbar/default");
	elm_object_item_part_content_set(navi_it, "toolbar_button2", btn2);
	evas_object_smart_callback_add(btn2, "clicked", _on_quit,
								sv);
}

void *memo_load_select_view(ug_data_t *ugd, service_h service)
{
    select_view_t *sv = SMALLOC(select_view_t);
    RETVIF(sv == NULL, NULL);
    sv->ugd = ugd;
    sv->is_textonly = service_key_check(service, "type", "textonly");

    /* date update hooker */
    memo_subscribe_change(_on_data_update, sv);

    /* init */
    memo_gl_itc_init(&sv->itc, "memo/edit/list");
    memo_gl_itc_init(&sv->itc_draw, "memo/edit/list_draw");

    /* create */
    _create_select_layout(sv);
    elm_object_part_content_set(ugd->base, "elm.swallow.content", sv->navigator);
    return (void *)sv;
}
